/*
 * 
 * Code reprit et adapté depuis avidemux
 *
 * FROM:
 * ----
 * (mean <fixounet@free.fr>)
 *
 * TO:
 * --
 * file      : gtkmarkscale.c
 * project   : xcfa
 * with      : Gtk-2
 *
 * copyright : (C) 2003,2004,2005,2006,2007,2008,2009 by Claude Bulin
 *
 * xcfa - GTK+ implementation of the GNU shell command
 * GNU General Public License
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * OLD ADRESS:
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * NEW ADRESS:
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
 
#include <math.h>
#include "gtkmarkscale.h"

static GtkWidgetClass *parent_class = NULL;

static gboolean gtk_markscale_expose (GtkWidget *widget, GdkEventExpose *event);

static void gtk_markscale_realize (GtkWidget *widget);

G_DEFINE_TYPE (GtkMarkScale, gtk_markscale, GTK_TYPE_SCALE)


gdouble ValueMarkA;
gdouble ValueMarkB;
guint   SecTime;

gdouble gtk_markscale_get_value_A (void)
{
	return ((gdouble)ValueMarkA);
}
gdouble gtk_markscale_get_value_B (void)
{
	return ((gdouble)ValueMarkB);
}

void gtk_markscale_set_SecTime (guint Sec)
{
	SecTime = Sec;
}
void gtk_markscale_set_value_A (GtkWidget *widget, gdouble A)
{
	GdkRectangle rect;
	
	ValueMarkA = A;
	
	rect.x      = widget->allocation.x;
	rect.y      = widget->allocation.y;
	rect.width  = widget->allocation.width;
	rect.height = widget->allocation.height;
	gdk_window_invalidate_rect(widget->window, &rect, FALSE);
}
void gtk_markscale_set_value_B (GtkWidget *widget, gdouble B)
{
	GdkRectangle rect;
	
	ValueMarkB = B;
	
	rect.x      = widget->allocation.x;
	rect.y      = widget->allocation.y;
	rect.width  = widget->allocation.width;
	rect.height = widget->allocation.height;
	gdk_window_invalidate_rect(widget->window, &rect, FALSE);
}

static void gtk_markscale_class_init (GtkMarkScaleClass *class)
{
	GtkWidgetClass *widget_class;
	GtkRangeClass *range_class;
	GtkScaleClass *scale_class;

	parent_class = g_type_class_peek_parent(class);
	widget_class = GTK_WIDGET_CLASS (class);
	range_class  = GTK_RANGE_CLASS (class);
	scale_class  = GTK_SCALE_CLASS (class);

	range_class->slider_detail = "hscale";

	widget_class->expose_event = gtk_markscale_expose;
	widget_class->realize = gtk_markscale_realize;
}

static void gtk_markscale_init (GtkMarkScale *markscale)
{
	GtkRange *range;

	range = GTK_RANGE (markscale);

	range->orientation = GTK_ORIENTATION_HORIZONTAL;
	range->flippable = TRUE;
	ValueMarkA = 0.0;
	ValueMarkB = 100.0;
	SecTime    = 0;
}

GtkWidget *gtk_markscale_new (GtkAdjustment *adjustment)
{
	return g_object_new (GTK_TYPE_MARKSCALE, "adjustment", adjustment, NULL);
}

static void gtk_markscale_realize (GtkWidget *widget)
{
	parent_class->realize(widget);
	gtk_widget_set_size_request(widget, widget->allocation.width, widget->allocation.height+10);
}

static gboolean gtk_markscale_expose (GtkWidget *widget, GdkEventExpose *event)
{
	GtkScale *scale;
  
	scale = GTK_SCALE (widget);

	if (GTK_WIDGET_CLASS (gtk_markscale_parent_class)->expose_event)
	    GTK_WIDGET_CLASS (gtk_markscale_parent_class)->expose_event (widget, event);

	GtkStateType state_type;
	state_type = GTK_STATE_NORMAL;
	if (!GTK_WIDGET_IS_SENSITIVE (scale)) state_type = GTK_STATE_INSENSITIVE;

	if (TRUE) {
	
		GdkGC    *gc = gdk_gc_new(widget->window);
		gint      width = widget->allocation.width;
		gint      start;
		gint      end;
		gint      top;
		gint      bottom;
		uint32_t  markA;
		uint32_t  markB;
		
		markA  = (int)floor((width-1)*((float)ValueMarkA / 100.0));
		markB  = (int)floor((width-1)*((float)ValueMarkB / 100.0));
		start  = widget->allocation.x + markA;
		end    = widget->allocation.x + markB;
		top    = widget->allocation.y +2;
		bottom = top + widget->allocation.height -5;
		
		/* mark A: */
		
		gdk_draw_line (widget->window,
					   gc,
					   start,
					   top,
					   start+4,
					   top);
		gdk_draw_line (widget->window,
					   gc,
					   start,
					   top,
					   start,
					   top+4);

		gdk_draw_line (widget->window,
					   gc,
					   start,
					   bottom,
					   start+4,
					   bottom);
		gdk_draw_line (widget->window,
					   gc,
					   start,
					   bottom - 4,
					   start,
					   bottom);
	
		/* mark B:  */
		
		gdk_draw_line (widget->window,
					   gc,
					   end,
					   top,
					   end-4,
					   top);
		gdk_draw_line (widget->window,
					   gc,
					   end,
					   top,
					   end,
					   top+4);

		gdk_draw_line (widget->window,
					   gc,
					   end,
					   bottom,
					   end-4,
					   bottom);
		gdk_draw_line (widget->window,
					   gc,
					   end,
					   bottom-4,
					   end,
					   bottom);
	}
	return FALSE;
}


#define __GTK_MARKSCALE_C__

